from mayan.apps.events.tests.mixins import EventTestCaseMixin
from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_workflow_edited
from ..models import WorkflowState
from ..permissions import permission_workflow_edit, permission_workflow_view

from .literals import (
    TEST_WORKFLOW_STATE_LABEL, TEST_WORKFLOW_STATE_COMPLETION,
)
from .mixins import WorkflowStateViewTestMixin, WorkflowTestMixin


class WorkflowStateViewTestCase(
    EventTestCaseMixin, WorkflowTestMixin, WorkflowStateViewTestMixin,
    GenericViewTestCase
):
    _test_event_object_name = 'test_workflow'

    def test_workflow_state_create_view_no_permission(self):
        self._create_test_workflow()
        self._clear_events()

        response = self._request_test_workflow_state_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowState.objects.count(), 0)
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_state_create_view_with_access(self):
        self._create_test_workflow()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_state_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowState.objects.count(), 1)
        self.assertEqual(
            WorkflowState.objects.all()[0].label, TEST_WORKFLOW_STATE_LABEL
        )
        self.assertEqual(
            WorkflowState.objects.all()[0].completion,
            TEST_WORKFLOW_STATE_COMPLETION
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_state_create_invalid_completion_view_with_access(self):
        self._create_test_workflow()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_state_create_view(
            extra_data={'completion': ''}
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowState.objects.count(), 1)
        self.assertEqual(
            WorkflowState.objects.all()[0].label, TEST_WORKFLOW_STATE_LABEL
        )
        self.assertEqual(
            WorkflowState.objects.all()[0].completion, 0
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_state_delete_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        response = self._request_test_workflow_state_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowState.objects.count(), 2)
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_state_delete_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_state_delete_view()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(WorkflowState.objects.count(), 1)
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_state_edit_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        workflow_state_label = self.test_workflow_state_1.label

        response = self._request_test_workflow_state_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_state_1.refresh_from_db()
        self.assertEqual(
            self.test_workflow_state_1.label, workflow_state_label
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_state_edit_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        workflow_state_label = self.test_workflow_state_1.label

        response = self._request_test_workflow_state_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_state_1.refresh_from_db()
        self.assertNotEquals(
            self.test_workflow_state_1.label, workflow_state_label
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_state_list_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        response = self._request_test_workflow_state_list_view()
        self.assertEqual(response.status_code, 404)

        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_state_list_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_state_list_view()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, text=self.test_workflow_state_1.label)

        event = self._get_test_object_event()
        self.assertEqual(event, None)



from rest_framework import status

from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models import Index, IndexTemplateNode
from ..permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_edit,
    permission_document_indexing_instance_view,
    permission_document_indexing_view
)

from .literals import TEST_INDEX_LABEL
from .mixins import (
    DocumentIndexAPIViewTestMixin, IndexInstanceAPIViewTestMixin,
    IndexInstanceNodeAPIViewTestMixin, IndexTestMixin,
    IndexTemplateAPIViewTestMixin, IndexTemplateNodeAPITestMixin
)


class DocumentIndexAPIViewTestCase(
    DocumentIndexAPIViewTestMixin, DocumentTestMixin, IndexTestMixin,
    BaseAPITestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_index()
        self._create_test_index_template_node(rebuild=True)

    def test_document_index_instance_list_api_view_no_permission(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_document_index_instance_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_document_index_instance_list_api_view_with_index_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_document_index_instance_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_document_index_instance_list_api_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_view
        )

        response = self._request_test_document_index_instance_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_document_index_instance_list_api_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_document_index_instance_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(
            response.data['results'][0]['id'], self.test_index_instance_node.pk
        )


class IndexInstanceAPIViewTestCase(
    IndexTestMixin, IndexInstanceAPIViewTestMixin, DocumentTestMixin,
    BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_index()

    def test_index_instance_detail_api_view_no_permission(self):
        response = self._request_test_index_instance_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('id' not in response.data)

    def test_index_instance_detail_api_view_with_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self.test_index.pk
        )

    def test_index_template_list_api_view_no_permission(self):
        response = self._request_test_index_instance_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_index_template_list_api_view_with_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(
            response.data['results'][0]['id'], self.test_index.pk
        )


class IndexInstanceNodeAPIViewTestCase(
    IndexTestMixin, IndexInstanceNodeAPIViewTestMixin, DocumentTestMixin,
    BaseAPITestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_index()
        self._create_test_index_template_node(rebuild=True)

    def test_index_instance_node_list_api_view_no_permission(self):
        response = self._request_test_index_instance_node_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_index_instance_node_list_api_view_with_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_node_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(
            response.data['results'][0]['id'],
            self.test_index_instance_node.pk
        )

    def test_index_instance_node_detail_api_view_no_permission(self):
        response = self._request_test_index_instance_node_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_index_instance_node_detail_api_view_with_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_node_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self.test_index_instance_node.pk
        )

    def test_index_instance_node_document_list_api_view_no_permission(self):
        response = self._request_test_index_instance_node_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_index_instance_node_document_list_api_view_with_index_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_node_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_index_instance_node_document_list_api_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_view
        )
        response = self._request_test_index_instance_node_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_index_instance_node_document_list_api_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_node_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(
            response.data['results'][0]['id'], self.test_document.pk
        )


class IndexTemplateAPIViewTestCase(
    IndexTestMixin, IndexTemplateAPIViewTestMixin, DocumentTestMixin,
    BaseAPITestCase
):
    auto_upload_test_document = False

    def test_index_template_create_api_view_no_permission(self):
        response = self._request_test_index_template_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(Index.objects.count(), 0)

    def test_index_template_create_api_view_with_permission(self):
        self.grant_permission(permission=permission_document_indexing_create)

        response = self._request_test_index_template_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        index = Index.objects.first()

        self.assertEqual(response.data['id'], index.pk)
        self.assertEqual(response.data['label'], index.label)

        self.assertEqual(Index.objects.count(), 1)
        self.assertEqual(index.label, TEST_INDEX_LABEL)

    def test_index_template_delete_api_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_template_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self.test_index in Index.objects.all())

    def test_index_template_delete_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_delete
        )

        response = self._request_test_index_template_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertTrue(self.test_index not in Index.objects.all())

    def test_index_template_detail_api_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_template_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('id' not in response.data)

    def test_index_template_detail_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_view
        )

        response = self._request_test_index_template_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self.test_index.pk
        )

    def test_index_template_edit_via_patch_api_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_template_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('id' not in response.data)

    def test_index_template_edit_via_patch_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )

        response = self._request_test_index_template_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self.test_index.pk
        )

    def test_index_template_list_api_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_template_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_index_template_list_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_view
        )

        response = self._request_test_index_template_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(
            response.data['results'][0]['id'], self.test_index.pk
        )


class IndexTemplateNodeAPIViewTestCase(
    IndexTestMixin, IndexTemplateNodeAPITestMixin, DocumentTestMixin,
    BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_index()

    def test_index_template_node_no_parent_create_api_view_no_permission(self):
        index_template_node_count = IndexTemplateNode.objects.count()
        response = self._request_test_index_template_node_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count
        )

    def test_index_template_node_no_parent_create_api_view_with_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        index_template_node_count = IndexTemplateNode.objects.count()

        response = self._request_test_index_template_node_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count
        )

    def test_index_template_node_bad_parent_create_api_view_no_permission(self):
        self._create_test_index()

        index_template_node_count = IndexTemplateNode.objects.count()
        response = self._request_test_index_template_node_create_api_view(
            extra_data={
                'parent': self.test_indexes[0].template_root.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count
        )

    def test_index_template_node_bad_parent_create_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        index_template_node_count = IndexTemplateNode.objects.count()

        response = self._request_test_index_template_node_create_api_view(
            extra_data={
                'parent': self.test_indexes[0].template_root.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count
        )

    def test_index_template_node_create_api_view_no_permission(self):
        index_template_node_count = IndexTemplateNode.objects.count()
        response = self._request_test_index_template_node_create_api_view(
            extra_data={
                'parent': self.test_index.template_root.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count
        )

    def test_index_template_node_create_api_view_with_access(self):
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        index_template_node_count = IndexTemplateNode.objects.count()

        response = self._request_test_index_template_node_create_api_view(
            extra_data={
                'parent': self.test_index.template_root.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(
            response.data['id'], self.test_index_template_node.pk
        )

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count + 1
        )

    def test_index_template_node_delete_api_view_no_permission(self):
        self._create_test_index_template_node()

        index_template_node_count = IndexTemplateNode.objects.count()
        response = self._request_test_index_template_node_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count
        )

    def test_index_template_node_delete_api_view_with_access(self):
        self._create_test_index_template_node()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        index_template_node_count = IndexTemplateNode.objects.count()

        response = self._request_test_index_template_node_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            IndexTemplateNode.objects.count(), index_template_node_count - 1
        )

    def test_index_template_node_detail_api_view_no_permission(self):
        self._create_test_index_template_node()

        response = self._request_test_index_template_node_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response.data)

    def test_index_template_node_detail_api_view_with_access(self):
        self._create_test_index_template_node()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_view
        )

        response = self._request_test_index_template_node_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self.test_index_template_node.pk
        )

    def test_index_template_node_edit_via_patch_api_view_no_permission(self):
        self._create_test_index_template_node()

        index_template_node_expression = self.test_index_template_node.expression

        response = self._request_test_index_template_node_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_index_template_node.refresh_from_db()
        self.assertEqual(
            index_template_node_expression,
            self.test_index_template_node.expression
        )

    def test_index_template_node_edit_via_patch_api_view_with_access(self):
        self._create_test_index_template_node()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        index_template_node_expression = self.test_index_template_node.expression

        response = self._request_test_index_template_node_edit_via_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_index_template_node.refresh_from_db()
        self.assertNotEqual(
            index_template_node_expression,
            self.test_index_template_node.expression
        )

    def test_index_template_node_list_api_view_no_permission(self):
        self._create_test_index_template_node()

        response = self._request_test_index_template_node_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('count' not in response)

    def test_index_template_node_list_api_view_with_access(self):
        self._create_test_index_template_node()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_view
        )

        response = self._request_test_index_template_node_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(
            response.data['results'][0]['id'],
            self.test_index_template_node.pk
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import document_page_search, document_search

document_search.add_model_field(
    field='versions__file_metadata_drivers__entries__key',
    label=_('File metadata key')
)
document_search.add_model_field(
    field='versions__file_metadata_drivers__entries__value',
    label=_('File metadata value')
)

document_page_search.add_model_field(
    field='document_version__file_metadata_drivers__entries__key',
    label=_('File metadata key')
)
document_page_search.add_model_field(
    field='document_version__file_metadata_drivers__entries__value',
    label=_('File metadata value')
)



from datetime import timedelta

from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_slow

from .literals import DELETE_STALE_UPLOADS_INTERVAL

queue_storage_periodic = CeleryQueue(
    label=_('Storage periodic'), name='storage_periodic', transient=True,
    worker=worker_slow
)
queue_storage_periodic.add_task_type(
    dotted_path='mayan.apps.storage.tasks.task_delete_stale_uploads',
    label=_('Delete stale uploads'), name='task_delete_stale_uploads',
    schedule=timedelta(
        seconds=DELETE_STALE_UPLOADS_INTERVAL
    )
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

namespace = SettingNamespace(label=_('Views'), name='views')

setting_paginate_by = namespace.add_setting(
    global_name='VIEWS_PAGINATE_BY',
    default=40,
    help_text=_(
        'The number objects that will be displayed per page.'
    )
)



DEFAULT_RESULTS_LIMIT = 100
# Duplicated to keep API compatible until version 4.0
# Merge these two literals and mixins on version 4.0
SEARCH_MODEL_NAME_API_KWARG = 'search_model'
SEARCH_MODEL_NAME_KWARG = 'search_model_name'
TASK_RETRY_DELAY = 5



from django.apps import apps
from django.db import connection
from django.utils.translation import ugettext_lazy as _

from mayan.apps.dynamic_search.classes import SearchModel
from mayan.apps.views.literals import LIST_MODE_CHOICE_ITEM

from .permissions import (
    permission_document_type_view, permission_document_view
)


def transformation_format_uuid(term_string):
    if connection.vendor in ('mysql', 'sqlite'):
        return term_string.replace('-', '')
    else:
        return term_string


def get_queryset_page_search_queryset():
    # Ignore documents in trash can
    DocumentPage = apps.get_model(
        app_label='documents', model_name='DocumentPage'
    )
    return DocumentPage.objects.filter(document_version__document__in_trash=False)


document_search = SearchModel(
    app_label='documents', list_mode=LIST_MODE_CHOICE_ITEM,
    model_name='Document', permission=permission_document_view,
    serializer_path='mayan.apps.documents.serializers.DocumentSerializer'
)

document_search.add_model_field(
    field='document_type__label', label=_('Document type label')
)
document_search.add_model_field(field='versions__mimetype')
document_search.add_model_field(field='label')
document_search.add_model_field(field='description')
document_search.add_model_field(
    field='uuid', transformation_function=transformation_format_uuid
)
document_search.add_model_field(field='versions__checksum')

document_page_search = SearchModel(
    app_label='documents', list_mode=LIST_MODE_CHOICE_ITEM,
    model_name='DocumentPage', permission=permission_document_view,
    queryset=get_queryset_page_search_queryset,
    serializer_path='mayan.apps.documents.serializers.DocumentPageSerializer'
)

document_page_search.add_model_field(
    field='document_version__document__document_type__label',
    label=_('Document type label')
)
document_page_search.add_model_field(
    field='document_version__document__versions__mimetype'
)
document_page_search.add_model_field(
    field='document_version__document__label'
)
document_page_search.add_model_field(
    field='document_version__document__description'
)
document_page_search.add_model_field(field='document_version__checksum')

document_type_search = SearchModel(
    app_label='documents', list_mode=LIST_MODE_CHOICE_ITEM,
    model_name='DocumentType', permission=permission_document_type_view,
    serializer_path='mayan.apps.documents.serializers.DocumentTypeSerializer'
)
document_type_search.add_model_field(field='label')



from unittest import skip

from selenium.common.exceptions import NoAlertPresentException

from django.conf import settings
from django.contrib.staticfiles.testing import StaticLiveServerTestCase

from mayan.apps.events.tests.mixins import EventTestCaseMixin
from mayan.apps.testing.tests.base import GenericViewTestCase
from mayan.apps.testing.tests.mixins import SeleniumTestMixin

from ..events import event_theme_created, event_theme_edited
from ..models import Theme
from ..permissions import (
    permission_theme_create, permission_theme_delete, permission_theme_edit,
    permission_theme_view
)

from .mixins import ThemeTestMixin, ThemeViewTestMixin


@skip('Skip until a synchronous live server class is added.')
class BasePlainViewTestCase(
    SeleniumTestMixin, StaticLiveServerTestCase, GenericViewTestCase
):
    auto_add_test_view = True
    test_view_url = r'^javascript:alert\("XSS"\)/$'
    test_view_is_public = True
    test_view_template = 'javascript_view'

    def test_login_view_url_fragment_xss(self):
        # Should redirect and not display an alert
        self._open_url(
            fragment='#javascript:alert("XSS")', viewname=settings.LOGIN_URL
        )

        with self.assertRaises(expected_exception=NoAlertPresentException):
            self.webdriver.switch_to_alert()

    def test_login_view_url_redirect(self):
        self._open_url(
            fragment='#javascript:alert("XSS")', viewname=settings.LOGIN_URL
        )

        self.assertTrue(self.test_view_template in self.webdriver.page_source)


class ThemeViewTestCase(
    EventTestCaseMixin, ThemeTestMixin, ThemeViewTestMixin, GenericViewTestCase
):
    _test_event_object_name = 'test_theme'

    def test_theme_create_view_no_permission(self):
        theme_count = Theme.objects.count()

        response = self._request_test_theme_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Theme.objects.count(), theme_count)

        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_theme_create_view_with_permissions(self):
        self.grant_permission(permission=permission_theme_create)

        theme_count = Theme.objects.count()

        response = self._request_test_theme_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Theme.objects.count(), theme_count + 1)

        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_theme_created.id)

    def test_theme_delete_view_no_permission(self):
        self._create_test_theme()

        theme_count = Theme.objects.count()

        response = self._request_test_theme_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Theme.objects.count(), theme_count)

        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_theme_created.id)

    def test_theme_delete_view_with_access(self):
        self._create_test_theme()

        self.grant_access(
            obj=self.test_theme, permission=permission_theme_delete
        )

        theme_count = Theme.objects.count()

        response = self._request_test_theme_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Theme.objects.count(), theme_count - 1)

        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_theme_edit_view_no_permission(self):
        self._create_test_theme()

        theme_label = self.test_theme.label

        response = self._request_test_theme_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_theme.refresh_from_db()
        self.assertEqual(self.test_theme.label, theme_label)

        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_theme_created.id)

    def test_theme_edit_view_with_access(self):
        self._create_test_theme()

        self.grant_access(
            obj=self.test_theme, permission=permission_theme_edit
        )

        theme_label = self.test_theme.label

        response = self._request_test_theme_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_theme.refresh_from_db()
        self.assertNotEqual(self.test_theme.label, theme_label)

        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_theme_edited.id)

    def test_theme_list_view_with_no_permission(self):
        self._create_test_theme()

        response = self._request_test_theme_list_view()
        self.assertNotContains(
            response=response, text=self.test_theme.label, status_code=200
        )

        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_theme_created.id)

    def test_theme_list_view_with_access(self):
        self._create_test_theme()

        self.grant_access(obj=self.test_theme, permission=permission_theme_view)

        response = self._request_test_theme_list_view()
        self.assertContains(
            response=response, text=self.test_theme.label, status_code=200
        )

        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_theme_created.id)



DEFAULT_STORAGE_BACKEND = 'django.core.files.storage.FileSystemStorage'
DELETE_STALE_UPLOADS_INTERVAL = 60 * 10  # 10 minutes
MSG_MIME_TYPES = (
    'application/vnd.ms-outlook', 'application/vnd.ms-office'
)
STORAGE_NAME_SHARED_UPLOADED_FILE = 'storage__shareduploadedfile'
UPLOAD_EXPIRATION_INTERVAL = 60 * 60 * 24 * 7  # 7 days



import logging

from django.contrib import messages
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.views.generics import (
    MultipleObjectConfirmActionView, SingleObjectCreateView,
    SingleObjectEditView, SingleObjectListView
)

from .icons import icon_message_list
from .links import link_message_create
from .models import Message
from .permissions import (
    permission_message_create, permission_message_delete,
    permission_message_edit, permission_message_view
)

logger = logging.getLogger(name=__name__)


class MessageCreateView(SingleObjectCreateView):
    fields = ('label', 'message', 'enabled', 'start_datetime', 'end_datetime')
    model = Message
    view_permission = permission_message_create

    def get_extra_context(self):
        return {
            'title': _('Create message'),
        }

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class MessageDeleteView(MultipleObjectConfirmActionView):
    model = Message
    object_permission = permission_message_delete
    pk_url_kwarg = 'message_id'
    post_action_redirect = reverse_lazy(viewname='motd:message_list')
    success_message = _('Delete request performed on %(count)d message')
    success_message_plural = _(
        'Delete request performed on %(count)d messages'
    )

    def get_extra_context(self):
        result = {
            'delete_view': True,
            'title': ungettext(
                singular='Delete the selected message?',
                plural='Delete the selected messages?',
                number=self.object_list.count()
            )
        }

        if self.object_list.count() == 1:
            result.update(
                {
                    'object': self.object_list.first(),
                    'title': _('Delete message: %s?') % self.object_list.first()
                }
            )

        return result

    def object_action(self, instance, form=None):
        try:
            instance.delete()
            messages.success(
                message=_(
                    'Message "%s" deleted successfully.'
                ) % instance, request=self.request
            )
        except Exception as exception:
            messages.error(
                message=_('Error deleting message "%(message)s": %(error)s') % {
                    'message': instance, 'error': exception
                }, request=self.request
            )


class MessageEditView(SingleObjectEditView):
    fields = ('label', 'message', 'enabled', 'start_datetime', 'end_datetime')
    model = Message
    object_permission = permission_message_edit
    pk_url_kwarg = 'message_id'
    post_action_redirect = reverse_lazy(viewname='motd:message_list')

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _('Edit message: %s') % self.object,
        }

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class MessageListView(SingleObjectListView):
    model = Message
    object_permission = permission_message_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_message_list,
            'no_results_main_link': link_message_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Messages are displayed in the login view. You can use '
                'messages to convery information about your organzation, '
                'announcements or usage guidelines for your users.'
            ),
            'no_results_title': _('No messages available'),
            'title': _('Messages'),
        }



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import document_search, document_page_search

document_search.add_model_field(
    field='versions__version_pages__ocr_content__content', label=_('OCR')
)

document_page_search.add_model_field(
    field='ocr_content__content', label=_('OCR')
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import menu_facet, menu_secondary, menu_tools

from .classes import SearchModel
from .links import (
    link_search, link_search_advanced, link_search_again,
    link_search_backend_reindex
)


class DynamicSearchApp(MayanAppConfig):
    app_namespace = 'search'
    app_url = 'search'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.dynamic_search'
    verbose_name = _('Dynamic search')

    def ready(self):
        super(DynamicSearchApp, self).ready()

        SearchModel.load_modules()
        SearchModel.initialize()

        menu_facet.bind_links(
            links=(link_search, link_search_advanced),
            sources=(
                'search:search', 'search:search_advanced', 'search:results'
            )
        )
        menu_secondary.bind_links(
            links=(link_search_again,), sources=('search:results',)
        )
        menu_tools.bind_links(
            links=(link_search_backend_reindex,),
        )



from django.core.exceptions import ImproperlyConfigured


class AsymmetricSerializerViewMixin:
    _write_methods = ('PATCH', 'POST', 'PUT')
    read_serializer_class = None
    write_serializer_class = None

    def get_read_serializer_class(self):
        if not self.read_serializer_class:
            raise ImproperlyConfigured(
                'View {} must specify a read_serializer_class.'.format(
                    self.__class__.__name__
                )
            )
        else:
            return self.read_serializer_class

    def get_serializer_class(self):
        if getattr(self, 'serializer_class', None):
            raise ImproperlyConfigured(
                'View {} can not use AsymmetricSerializerViewMixin while '
                'also specifying a serializer_class.'.format(
                    self.__class__.__name__
                )
            )

        if self.request.method in self._write_methods:
            return self.get_write_serializer_class()
        else:
            return self.get_read_serializer_class()

    def get_write_serializer_class(self):
        if not self.write_serializer_class:
            raise ImproperlyConfigured(
                'View {} must specify a write_serializer_class.'.format(
                    self.__class__.__name__
                )
            )
        else:
            return self.write_serializer_class



import logging

from celery import chain, chord

from django.apps import apps
from django.db import OperationalError, transaction

from mayan.apps.documents.tasks import task_generate_document_page_image
from mayan.celery import app

from .events import event_ocr_document_version_finish
from .literals import DO_OCR_RETRY_DELAY
from .signals import signal_post_document_version_ocr

logger = logging.getLogger(name=__name__)


@app.task(bind=True, default_retry_delay=DO_OCR_RETRY_DELAY, ignore_result=True)
def task_document_version_process(self, document_version_pk):
    logger.info('Starting OCR for document version page: %s', document_version_pk)
    DocumentVersion = apps.get_model(
        app_label='documents', model_name='DocumentVersion'
    )

    document_version = DocumentVersion.objects.get(
        pk=document_version_pk
    )

    try:
        page_tasks = []
        for document_version_page in document_version.pages.all():
            page_tasks.append(
                chain(
                    task_generate_document_page_image.s(document_page_id=document_version_page.pk),
                    task_document_version_page_process_ocr.s(document_page_pk=document_version_page.pk)
                )
            )
        chord(page_tasks)(task_document_version_finished.s(document_version_pk=document_version.pk))
    except Exception as exception:
        document_version.ocr_errors.create(result=exception)
        raise
    except OperationalError as exception:
        logger.warning(
            'OCR operational error for document version: %d; %s. Retrying.',
            document_version_pk, exception
        )
        raise self.retry(exc=exception)


@app.task(bind=True, default_retry_delay=DO_OCR_RETRY_DELAY, ignore_result=True)
def task_document_version_page_process_ocr(self, cache_filename, document_page_pk):
    DocumentPageOCRContent = apps.get_model(
        app_label='ocr', model_name='DocumentPageOCRContent'
    )
    DocumentPage = apps.get_model(
        app_label='documents', model_name='DocumentPage'
    )
    document_page = DocumentPage.objects.get(pk=document_page_pk)

    try:
        DocumentPageOCRContent.objects.process_document_page(
            cache_filename=cache_filename, document_page=document_page,
        )
    except OperationalError as exception:
        raise self.retry(exc=exception)


@app.task(bind=True, default_retry_delay=DO_OCR_RETRY_DELAY, ignore_result=True)
def task_document_version_finished(self, results, document_version_pk):
    logger.info(
        'OCR complete for document version: %s', document_version_pk
    )

    DocumentVersion = apps.get_model(
        app_label='documents', model_name='DocumentVersion'
    )
    document_version = DocumentVersion.objects.get(pk=document_version_pk)

    document_version.ocr_errors.all().delete()

    try:
        with transaction.atomic():
            event_ocr_document_version_finish.commit(
                action_object=document_version.document,
                target=document_version
            )

            transaction.on_commit(
                lambda: signal_post_document_version_ocr.send(
                    sender=document_version.__class__,
                    instance=document_version
                )
            )
    except Exception as exception:
        logger.error(
            'Exception in OCR finish for document version: %d; %s', document_version_pk,
            exception
        )
        document_version.ocr_errors.create(result=exception)
        raise
    except OperationalError as exception:
        logger.warning(
            'Operational error in OCR finish for document version: %d; %s. Retrying.',
            document_version, exception
        )
        raise self.retry(exc=exception)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import document_page_search, document_search

document_page_search.add_model_field(
    field='document_version__document__comments__comment',
    label=_('Comments')
)
document_search.add_model_field(
    field='comments__comment',
    label=_('Comments')
)



import uuid

from django.db import models
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from .classes import DefinedStorageLazy
from .literals import STORAGE_NAME_SHARED_UPLOADED_FILE


def upload_to(instance, filename):
    return 'shared-file-{}'.format(uuid.uuid4().hex)


class SharedUploadedFile(models.Model):
    """
    Keep a database link to a stored file. Used to share files between code
    that runs out of process.
    """
    file = models.FileField(
        storage=DefinedStorageLazy(
            name=STORAGE_NAME_SHARED_UPLOADED_FILE
        ), upload_to=upload_to, verbose_name=_('File')
    )
    filename = models.CharField(max_length=255, verbose_name=_('Filename'))
    datetime = models.DateTimeField(
        auto_now_add=True, verbose_name=_('Date time')
    )

    class Meta:
        verbose_name = _('Shared uploaded file')
        verbose_name_plural = _('Shared uploaded files')

    def __str__(self):
        return self.filename

    def delete(self, *args, **kwargs):
        self.file.storage.delete(name=self.file.name)
        return super().delete(*args, **kwargs)

    def open(self):
        return self.file.storage.open(name=self.file.name)

    def save(self, *args, **kwargs):
        self.filename = force_text(self.file)
        super().save(*args, **kwargs)



import os

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

from .literals import LOGGING_HANDLER_OPTIONS

namespace = SettingNamespace(label=_('Logging'), name='logging')

setting_logging_enable = namespace.add_setting(
    global_name='LOGGING_ENABLE', default=True, help_text=_(
        'Automatically enable logging to all apps.'
    )
)
setting_logging_handlers = namespace.add_setting(
    global_name='LOGGING_HANDLERS', default=('console',),
    help_text=_(
        'List of handlers to which logging messages will be sent. '
        'Options are: {}'.format(', '.join(LOGGING_HANDLER_OPTIONS))
    )
)
setting_logging_level = namespace.add_setting(
    global_name='LOGGING_LEVEL', default='ERROR', help_text=_(
        'Level for the logging system.'
    )
)
setting_logging_log_file_path = namespace.add_setting(
    global_name='LOGGING_LOG_FILE_PATH',
    default=os.path.join(settings.MEDIA_ROOT, 'error.log'), help_text=_(
        'Path to the logfile that will track errors during production.'
    ),
    is_path=True
)



from django.conf.urls import url

from .api_views import APIMessageListView, APIMessageView
from .views import (
    MessageCreateView, MessageDeleteView, MessageEditView, MessageListView
)

urlpatterns = [
    url(
        regex=r'^messages/$', name='message_list',
        view=MessageListView.as_view()
    ),
    url(
        regex=r'^messages/create/$', name='message_create',
        view=MessageCreateView.as_view()
    ),
    url(
        regex=r'^messages/(?P<message_id>\d+)/delete/$',
        name='message_single_delete', view=MessageDeleteView.as_view()
    ),
    url(
        regex=r'^messages/multiple/delete/$',
        name='message_multiple_delete', view=MessageDeleteView.as_view()
    ),
    url(
        regex=r'^messages/(?P<message_id>\d+)/edit/$', name='message_edit',
        view=MessageEditView.as_view()
    ),
]

api_urls = [
    url(
        regex=r'^messages/$', name='message-list',
        view=APIMessageListView.as_view()
    ),
    url(
        regex=r'^messages/(?P<pk>[0-9]+)/$', name='message-detail',
        view=APIMessageView.as_view()
    ),
]



from ..models import Comment

from .literals import TEST_COMMENT_TEXT, TEST_COMMENT_TEXT_EDITED


class CommentAPIViewTestMixin:
    def _request_test_comment_create_api_view(self):
        pk_list = list(Comment.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='rest_api:comment-list', kwargs={
                'document_pk': self.test_document.pk
            }, data={
                'comment': TEST_COMMENT_TEXT
            }
        )

        self.test_document_comment = Comment.objects.exclude(
            pk__in=pk_list
        ).first()

        return response

    def _request_test_comment_delete_api_view(self):
        return self.delete(
            viewname='rest_api:comment-detail', kwargs={
                'document_pk': self.test_document.pk,
                'comment_pk': self.test_document_comment.pk,
            }
        )

    def _request_test_comment_detail_api_view(self):
        return self.get(
            viewname='rest_api:comment-detail', kwargs={
                'document_pk': self.test_document.pk,
                'comment_pk': self.test_document_comment.pk
            }
        )

    def _request_test_comment_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:comment-detail', kwargs={
                'document_pk': self.test_document.pk,
                'comment_pk': self.test_document_comment.pk,
            }, data={'comment': TEST_COMMENT_TEXT_EDITED}
        )

    def _request_test_comment_list_api_view(self):
        return self.get(
            viewname='rest_api:comment-list', kwargs={
                'document_pk': self.test_document.pk
            }
        )


class DocumentCommentTestMixin:
    def _create_test_comment(self):
        self.test_document_comment = self.test_document.comments.create(
            comment=TEST_COMMENT_TEXT, user=self.test_user
        )


class DocumentCommentViewTestMixin:
    def _request_test_comment_create_view(self):
        return self.post(
            viewname='comments:comment_add', kwargs={
                'document_id': self.test_document.pk
            }, data={'comment': TEST_COMMENT_TEXT}
        )

    def _request_test_comment_delete_view(self):
        return self.post(
            viewname='comments:comment_delete', kwargs={
                'comment_id': self.test_document_comment.pk
            },
        )

    def _request_test_comment_edit_view(self):
        return self.post(
            viewname='comments:comment_edit', kwargs={
                'comment_id': self.test_document_comment.pk,
            }, data={
                'comment': TEST_COMMENT_TEXT_EDITED
            }
        )

    def _request_test_comment_list_view(self):
        return self.get(
            viewname='comments:comments_for_document', kwargs={
                'document_id': self.test_document.pk,
            }
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import menu_object, menu_secondary, menu_tools
from mayan.apps.navigation.classes import SourceColumn

from .classes import StatisticLineChart, StatisticNamespace
from .links import (
    link_execute, link_namespace_details, link_namespace_list,
    link_statistics, link_view
)


class StatisticsApp(MayanAppConfig):
    app_namespace = 'statistics'
    app_url = 'statistics'
    has_static_media = True
    has_tests = True
    name = 'mayan.apps.mayan_statistics'
    static_media_ignore_patterns = (
        'statistics/node_modules/chart.js/book.*',
        'statistics/node_modules/chart.js/karma.conf.*',
        'statistics/node_modules/chart.js/samples/*',
        'statistics/node_modules/chart.js/src/*',
        'statistics/node_modules/chart.js/*docs*',
    )
    verbose_name = _('Statistics')

    def ready(self):
        super(StatisticsApp, self).ready()

        SourceColumn(
            attribute='schedule',
            # Translators: Schedule here is a noun, the 'schedule' at
            # which the statistic will be updated
            include_label=True, label=_('Schedule'),
            source=StatisticLineChart
        )

        SourceColumn(
            attribute='get_last_update', include_label=True,
            label=_('Last update'), source=StatisticLineChart
        )

        menu_object.bind_links(
            links=(link_execute, link_view), sources=(StatisticLineChart,)
        )
        menu_object.bind_links(
            links=(link_namespace_details,), sources=(StatisticNamespace,)
        )
        menu_secondary.bind_links(
            links=(link_namespace_list,),
            sources=(StatisticNamespace, 'statistics:namespace_list')
        )
        menu_tools.bind_links(links=(link_statistics,))



from importlib import import_module
import logging

from django.apps import apps
from django.utils.encoding import force_text

logger = logging.getLogger(name=__name__)


class AppsModuleLoaderMixin:
    _loader_module_name = None
    _loader_module_sets = {}

    @classmethod
    def load_modules(cls):
        # This set keeps track of what apps have already been processed.
        cls._loader_module_sets.setdefault(cls._loader_module_name, set())

        for app in apps.get_app_configs():
            if app not in cls._loader_module_sets[cls._loader_module_name]:
                try:
                    import_module('{}.{}'.format(app.name, cls._loader_module_name))
                except ImportError as exception:
                    non_fatal_messages = (
                        'No module named {module_name}'.format(module_name=cls._loader_module_name),
                        'No module named \'{app_label}.{module_name}\''.format(app_label=app.name, module_name=cls._loader_module_name)
                    )
                    if force_text(exception) not in non_fatal_messages:
                        logger.error(
                            'Error importing %s %s.py file; %s', app.name,
                            cls._loader_module_name, exception
                        )
                        raise
                finally:
                    cls._loader_module_sets[cls._loader_module_name].add(app)



from django.contrib.contenttypes.models import ContentType
from django.core.files import File

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.permissions.tests.mixins import PermissionTestMixin

from ..classes import Layer
from ..models import Asset, ObjectLayer
from ..transformations import BaseTransformation

from .literals import (
    TEST_ASSET_LABEL, TEST_ASSET_LABEL_EDITED, TEST_ASSET_INTERNAL_NAME,
    TEST_ASSET_PATH, TEST_TRANSFORMATION_CLASS_LABEL,
    TEST_TRANSFORMATION_CLASS_NAME, TEST_TRANSFORMATION_NAME,
    TEST_TRANSFORMATION_ARGUMENT, TEST_TRANSFORMATION_ARGUMENT_EDITED
)


class AssetTestMixin:
    def _create_test_asset(self):
        with open(file=TEST_ASSET_PATH, mode='rb') as file_object:
            self.test_asset = Asset.objects.create(
                label=TEST_ASSET_LABEL,
                internal_name=TEST_ASSET_INTERNAL_NAME,
                file=File(file_object)
            )


class AssetViewTestMixin:
    def _request_test_asset_create_view(self):
        pk_list = list(Asset.objects.values_list('pk', flat=True))

        with open(file=TEST_ASSET_PATH, mode='rb') as file_object:
            response = self.post(
                viewname='converter:asset_create', data={
                    'label': TEST_ASSET_LABEL,
                    'internal_name': TEST_ASSET_INTERNAL_NAME,
                    'file': file_object
                }
            )

        self.test_asset = Asset.objects.exclude(
            pk__in=pk_list
        ).first()

        return response

    def _request_test_asset_delete_view(self):
        return self.post(
            viewname='converter:asset_single_delete', kwargs={
                'asset_id': self.test_asset.pk
            }
        )

    def _request_test_asset_edit_view(self):
        return self.post(
            viewname='converter:asset_edit', kwargs={
                'asset_id': self.test_asset.pk
            }, data={
                'label': TEST_ASSET_LABEL_EDITED,
                'internal_name': TEST_ASSET_INTERNAL_NAME,
            }
        )

    def _request_test_asset_list_view(self):
        return self.get(viewname='converter:asset_list')


class LayerTestCaseMixin:
    def setUp(self):
        super(LayerTestCaseMixin, self).setUp()
        Layer.invalidate_cache()


class LayerTestMixin(PermissionTestMixin):
    test_layer = Layer(
        label='Test layer', name='test_layer', order=1000,
        permissions={}
    )

    def setUp(self):
        super(LayerTestMixin, self).setUp()
        self._create_test_permission()

        self.test_layer_permission = self.test_permission
        ModelPermission.register(
            model=self.test_document._meta.model, permissions=(
                self.test_permission,
            )
        )

        self.test_layer.permissions = {
            'create': self.test_layer_permission,
            'delete': self.test_layer_permission,
            'edit': self.test_layer_permission,
            'select': self.test_layer_permission,
            'view': self.test_layer_permission,
        }


class TransformationTestMixin(LayerTestMixin):
    def _create_test_transformation(self):
        content_type = ContentType.objects.get_for_model(
            model=self.test_document
        )
        object_layer, created = ObjectLayer.objects.get_or_create(
            content_type=content_type, object_id=self.test_document.pk,
            stored_layer=self.test_layer.stored_layer
        )

        self.test_transformation = object_layer.transformations.create(
            name=TEST_TRANSFORMATION_NAME,
            arguments=TEST_TRANSFORMATION_ARGUMENT
        )

    def _create_test_transformation_class(self):
        class TestTransformation(BaseTransformation):
            arguments = ('test_argument',)
            label = TEST_TRANSFORMATION_CLASS_LABEL
            name = TEST_TRANSFORMATION_CLASS_NAME

            def execute_on(self, *args, **kwargs):
                super(TestTransformation, self).execute_on(*args, **kwargs)

                return self.image

        BaseTransformation.register(
            layer=self.test_layer, transformation=TestTransformation
        )

        self.TestTransformationClass = TestTransformation


class TransformationViewsTestMixin:
    def _request_transformation_create_view(self):
        return self.post(
            viewname='converter:transformation_create', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name,
                'transformation_name': TEST_TRANSFORMATION_NAME,
            }, data={
                'arguments': TEST_TRANSFORMATION_ARGUMENT
            }
        )

    def _request_transformation_delete_view(self):
        return self.post(
            viewname='converter:transformation_delete', kwargs={
                'layer_name': self.test_layer.name,
                'transformation_id': self.test_transformation.pk
            }
        )

    def _request_transformation_edit_view(self):
        return self.post(
            viewname='converter:transformation_edit', kwargs={
                'layer_name': self.test_layer.name,
                'transformation_id': self.test_transformation.pk
            }, data={
                'arguments': TEST_TRANSFORMATION_ARGUMENT_EDITED
            }
        )

    def _request_transformation_list_view(self):
        return self.get(
            viewname='converter:transformation_list', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name
            }
        )

    def _request_test_transformation_select_get_view(self):
        return self.get(
            viewname='converter:transformation_select', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name
            }
        )

    def _request_test_transformation_select_post_view(self):
        return self.post(
            viewname='converter:transformation_select', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name
            }, data={
                'transformation': TEST_TRANSFORMATION_CLASS_NAME
            }
        )



from mayan.apps.common.tests.mixins import ObjectCopyTestMixin
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from .mixins import WorkflowStateActionTestMixin, WorkflowTestMixin


class WorkflowTemplateCopyTestCase(
    DocumentTestMixin, ObjectCopyTestMixin, WorkflowStateActionTestMixin,
    WorkflowTestMixin, BaseTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._create_test_workflow_state_action()
        self.test_workflow.document_types.add(self.test_document_type)
        self.test_object = self.test_workflow



from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.events.tests.mixins import EventTestCaseMixin
from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_workflow_edited
from ..literals import WIDGET_CLASS_TEXTAREA
from ..models import WorkflowTransition
from ..permissions import (
    permission_workflow_edit, permission_workflow_transition,
    permission_workflow_view
)

from .literals import (
    TEST_WORKFLOW_TRANSITION_LABEL, TEST_WORKFLOW_TRANSITION_LABEL_EDITED
)
from .mixins import (
    WorkflowTestMixin, WorkflowTransitionEventViewTestMixin,
    WorkflowTransitionFieldTestMixin, WorkflowTransitionFieldViewTestMixin,
    WorkflowViewTestMixin, WorkflowTransitionViewTestMixin
)


class WorkflowTransitionViewTestCase(
    EventTestCaseMixin, WorkflowTestMixin, WorkflowViewTestMixin,
    WorkflowTransitionViewTestMixin, GenericViewTestCase
):
    _test_event_object_name = 'test_workflow'

    def test_workflow_transition_create_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        response = self._request_test_workflow_transition_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowTransition.objects.count(), 0)
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_create_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowTransition.objects.count(), 1)
        self.assertEqual(
            WorkflowTransition.objects.all()[0].label,
            TEST_WORKFLOW_TRANSITION_LABEL
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].origin_state,
            self.test_workflow_state_1
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].destination_state,
            self.test_workflow_state_2
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_transition_delete_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._clear_events()

        response = self._request_test_workflow_transition_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self.test_workflow_transition in WorkflowTransition.objects.all()
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_delete_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._clear_events()

        self.grant_access(permission=permission_workflow_edit, obj=self.test_workflow)

        response = self._request_test_workflow_transition_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self.test_workflow_transition in WorkflowTransition.objects.all()
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_transition_edit_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._clear_events()

        response = self._request_test_workflow_transition_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_transition.refresh_from_db()
        self.assertEqual(
            self.test_workflow_transition.label, TEST_WORKFLOW_TRANSITION_LABEL
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_edit_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_transition.refresh_from_db()
        self.assertEqual(
            self.test_workflow_transition.label,
            TEST_WORKFLOW_TRANSITION_LABEL_EDITED
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_transition_list_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._clear_events()

        response = self._request_test_workflow_transition_list_view()
        self.assertNotContains(
            response=response, text=self.test_workflow_transition.label,
            status_code=404
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_list_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_transition_list_view()
        self.assertContains(
            response=response, text=self.test_workflow_transition.label,
            status_code=200
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)


class WorkflowTransitionDocumentViewTestCase(
    WorkflowTestMixin, WorkflowViewTestMixin, WorkflowTransitionViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super(WorkflowTransitionDocumentViewTestCase, self).setUp()
        self._create_test_workflow()
        self.test_workflow.document_types.add(self.test_document_type)
        self._create_test_workflow_states()
        self._create_test_workflow_transitions()
        self._upload_test_document()
        self.test_workflow_instance = self.test_document.workflows.first()

    def test_workflow_transition_selection_get_view_with_workflow_access(self):
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_transition
        )
        response = self._request_test_workflow_transition_selection_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_1
        )

    def test_workflow_transition_selection_post_view_with_workflow_access(self):
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_transition
        )
        response = self._request_test_workflow_transition_selection_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_1
        )

    def test_workflow_transition_execute_view_no_permission(self):
        """
        Test transitioning a workflow without the transition workflow
        permission.
        """
        response = self._request_test_workflow_transition_execute_view()
        self.assertEqual(response.status_code, 404)

        # Workflow should remain in the same initial state
        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_1
        )

    def test_workflow_transition_execute_view_with_workflow_access(self):
        """
        Test transitioning a workflow by granting the transition workflow
        permission to the role.
        """
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_transition
        )

        response = self._request_test_workflow_transition_execute_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_2
        )

    def test_workflow_transition_execute_view_with_transition_access(self):
        """
        Test transitioning a workflow by granting the transition workflow
        permission to the role.
        """
        self.grant_access(
            obj=self.test_workflow_transition,
            permission=permission_workflow_transition
        )

        response = self._request_test_workflow_transition_execute_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_2
        )


class WorkflowTransitionEventViewTestCase(
    WorkflowTestMixin, WorkflowTransitionEventViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_workflow_transition_event_list_view_no_permission(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        response = self._request_test_workflow_transition_event_list_view()
        self.assertEqual(response.status_code, 404)

    def test_workflow_transition_event_list_view_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_event_list_view()
        self.assertEqual(response.status_code, 200)


class WorkflowTransitionFieldViewTestCase(
    EventTestCaseMixin, WorkflowTestMixin, WorkflowTransitionFieldTestMixin,
    WorkflowTransitionFieldViewTestMixin, WorkflowTransitionViewTestMixin,
    GenericViewTestCase
):
    _test_event_object_name = 'test_workflow'

    def setUp(self):
        super(WorkflowTransitionFieldViewTestCase, self).setUp()
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

    def test_workflow_transition_field_create_view_no_permission(self):
        workflow_transition_field_count = self.test_workflow_transition.fields.count()
        self._clear_events()

        response = self._request_workflow_transition_field_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_field_create_view_with_access(self):
        workflow_transition_field_count = self.test_workflow_transition.fields.count()
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )
        self._clear_events()

        response = self._request_workflow_transition_field_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count + 1
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_transition_field_delete_view_no_permission(self):
        self._create_test_workflow_transition_field()
        workflow_transition_field_count = self.test_workflow_transition.fields.count()
        self._clear_events()

        response = self._request_workflow_transition_field_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_field_delete_view_with_access(self):
        self._create_test_workflow_transition_field()
        workflow_transition_field_count = self.test_workflow_transition.fields.count()
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_workflow_transition_field_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count - 1
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_transition_field_edit_view_no_permission(self):
        self._create_test_workflow_transition_field()
        workflow_transition_field_label = self.test_workflow_transition_field.label
        self._clear_events()

        response = self._request_workflow_transition_field_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_transition_field.refresh_from_db()
        self.assertEqual(
            workflow_transition_field_label,
            self.test_workflow_transition_field.label
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_field_edit_view_with_access(self):
        self._create_test_workflow_transition_field()
        workflow_transition_field_label = self.test_workflow_transition_field.label
        self._clear_events()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_workflow_transition_field_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_transition_field.refresh_from_db()
        self.assertNotEqual(
            workflow_transition_field_label,
            self.test_workflow_transition_field.label
        )
        event = self._get_test_object_event()
        self.assertEqual(event.verb, event_workflow_edited.id)
        self.assertEqual(event.actor, self._test_case_user)

    def test_workflow_transition_field_list_view_no_permission(self):
        self._create_test_workflow_transition_field()
        self._clear_events()

        response = self._request_test_workflow_transition_field_list_view()
        self.assertNotContains(
            response=response,
            text=self.test_workflow_transition_field.label,
            status_code=404
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)

    def test_workflow_transition_field_list_view_with_access(self):
        self._create_test_workflow_transition_field()
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )
        self._clear_events()

        response = self._request_test_workflow_transition_field_list_view()
        self.assertContains(
            response=response,
            text=self.test_workflow_transition_field.label,
            status_code=200
        )
        event = self._get_test_object_event()
        self.assertEqual(event, None)


class WorkflowInstanceTransitionFieldViewTestCase(
    WorkflowTestMixin, WorkflowTransitionFieldTestMixin,
    WorkflowTransitionFieldViewTestMixin, WorkflowTransitionViewTestMixin,
    GenericDocumentViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_workflow(add_document_type=True)
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self._create_test_workflow_transition_field(
            extra_data={
                'widget': WIDGET_CLASS_TEXTAREA
            }
        )
        self._upload_test_document()
        self.test_workflow_instance = self.test_document.workflows.first()

    def test_workflow_transition_text_area_widget_execute_view_with_transition_access(self):
        self.grant_access(
            obj=self.test_workflow_transition,
            permission=permission_workflow_transition
        )

        response = self._request_test_workflow_transition_execute_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_2
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_current_user_theme_settings_details,
    icon_current_user_theme_settings_edit
)
from .permissions import (
    permission_theme_create, permission_theme_delete, permission_theme_edit,
)

link_current_user_theme_settings_details = Link(
    icon_class=icon_current_user_theme_settings_details,
    text=_('Theme settings'),
    view='appearance:current_user_theme_settings_details'
)
link_current_user_theme_settings_edit = Link(
    icon_class=icon_current_user_theme_settings_edit,
    text=_('Edit theme settings'),
    view='appearance:current_user_theme_settings_edit'
)

link_theme_create = Link(
    icon_class_path='mayan.apps.appearance.icons.icon_theme_create',
    permissions=(permission_theme_create,),
    text=_('Create new theme'), view='appearance:theme_create'
)
link_theme_delete = Link(
    args='object.pk',
    icon_class_path='mayan.apps.appearance.icons.icon_theme_delete',
    permissions=(permission_theme_delete,),
    tags='dangerous', text=_('Delete'), view='appearance:theme_delete',
)
link_theme_edit = Link(
    args='object.pk',
    icon_class_path='mayan.apps.appearance.icons.icon_theme_edit',
    permissions=(permission_theme_edit,),
    text=_('Edit'), view='appearance:theme_edit',
)
link_theme_list = Link(
    icon_class_path='mayan.apps.appearance.icons.icon_theme_list',
    text=_('Themes'), view='appearance:theme_list'
)
link_theme_setup = Link(
    icon_class_path='mayan.apps.appearance.icons.icon_theme_setup',
    permissions=(permission_theme_create,), text=_('Themes'),
    view='appearance:theme_list'
)



from ..models import Message

from .literals import (
    TEST_LABEL, TEST_LABEL_EDITED, TEST_MESSAGE, TEST_MESSAGE_EDITED
)


class MessageAPIViewTestMixin:
    def _request_message_create_view(self):
        return self.post(
            viewname='rest_api:message-list', data={
                'label': TEST_LABEL, 'message': TEST_MESSAGE
            }
        )

    def _request_message_delete_view(self):
        return self.delete(
            viewname='rest_api:message-detail', kwargs={
                'pk': self.test_message.pk
            }
        )

    def _request_message_detail_view(self):
        return self.get(
            viewname='rest_api:message-detail', kwargs={
                'pk': self.test_message.pk
            }
        )

    def _request_message_edit_via_patch_view(self):
        return self.patch(
            viewname='rest_api:message-detail', kwargs={
                'pk': self.test_message.pk
            }, data={
                'label': TEST_LABEL_EDITED,
                'message': TEST_MESSAGE_EDITED
            }
        )

    def _request_message_edit_via_put_view(self):
        return self.put(
            viewname='rest_api:message-detail', kwargs={
                'pk': self.test_message.pk
            }, data={
                'label': TEST_LABEL_EDITED,
                'message': TEST_MESSAGE_EDITED
            }
        )


class MessageTestMixin:
    def _create_test_message(self):
        self.test_message = Message.objects.create(
            label=TEST_LABEL, message=TEST_MESSAGE
        )


class MessageViewTestMixin:
    def _request_test_message_create_view(self):
        pk_list = list(Message.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='motd:message_create', data={
                'label': TEST_LABEL,
                'message': TEST_MESSAGE
            }
        )

        self.test_message = Message.objects.exclude(
            pk__in=pk_list
        ).first()

        return response

    def _request_test_message_delete_view(self):
        return self.post(
            viewname='motd:message_single_delete', kwargs={
                'message_id': self.test_message.pk
            }
        )

    def _request_test_message_edit_view(self):
        return self.post(
            viewname='motd:message_edit', kwargs={
                'message_id': self.test_message.pk
            }, data={
                'label': TEST_MESSAGE_EDITED,
                'message': TEST_MESSAGE_EDITED
            }
        )

    def _request_test_message_list_view(self):
        return self.get(viewname='motd:message_list')



from django.utils.encoding import force_text

from mayan.apps.testing.tests.base import GenericViewTestCase

from ..models import AccessControlList
from ..permissions import permission_acl_edit, permission_acl_view

from .mixins import ACLTestMixin, AccessControlListViewTestMixin


class AccessControlListViewTestCase(
    AccessControlListViewTestMixin, ACLTestMixin, GenericViewTestCase
):
    def test_acl_create_get_view_no_permission(self):
        self._create_acl_test_object()

        acl_count = AccessControlList.objects.count()

        response = self._request_test_acl_create_get_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(AccessControlList.objects.count(), acl_count)

    def test_acl_create_get_view_with_access(self):
        self._create_acl_test_object()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )
        acl_count = AccessControlList.objects.count()

        response = self._request_test_acl_create_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(AccessControlList.objects.count(), acl_count)

    def test_acl_create_view_post_no_permission(self):
        self._create_acl_test_object()

        acl_count = AccessControlList.objects.count()

        response = self._request_test_acl_create_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(AccessControlList.objects.count(), acl_count)

    def test_acl_create_view_post_with_access(self):
        self._create_acl_test_object()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        acl_count = AccessControlList.objects.count()

        response = self._request_test_acl_create_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(AccessControlList.objects.count(), acl_count + 1)

    def test_acl_delete_view_no_permission(self):
        self._create_acl_test_object()

        self._create_test_acl()

        response = self._request_test_acl_delete_view()
        self.assertNotContains(
            response=response, text=force_text(s=self.test_object),
            status_code=404
        )

        self.assertTrue(
            self.test_object.acls.filter(role=self.test_role).exists()
        )

    def test_acl_delete_view_with_access(self):
        self._create_acl_test_object()

        self._create_test_acl()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        response = self._request_test_acl_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self.test_object.acls.filter(role=self.test_role).exists()
        )

    def test_acl_list_view_no_permission(self):
        self._create_acl_test_object()

        self._create_test_acl()

        response = self._request_test_acl_list_view()

        self.assertNotContains(
            response=response, text=force_text(s=self.test_object),
            status_code=404
        )

    def test_acl_list_view_with_access(self):
        self._create_acl_test_object()

        self._create_test_acl()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_view
        )

        response = self._request_test_acl_list_view()

        self.assertContains(
            response=response, text=force_text(s=self.test_object),
            status_code=200
        )

    def test_acl_permission_get_no_permission(self):
        self._create_acl_test_object()

        self._create_test_acl()

        response = self._request_test_acl_permission_list_get_view()

        self.assertNotContains(
            response=response, text=force_text(s=self.test_object),
            status_code=404
        )

    def test_acl_permission_get_with_access(self):
        self._create_acl_test_object()

        self._create_test_acl()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        response = self._request_test_acl_permission_list_get_view()

        self.assertContains(
            response=response, text=force_text(s=self.test_object),
            status_code=200
        )

    def test_orphan_acl_create_view_with_permission(self):
        """
        Test creating an ACL entry for an object with no model permissions.
        Result: Should display a blank permissions list (not optgroup)
        """
        self._create_acl_test_object(
            create_test_permission=False, register_model_permissions=False
        )

        self.grant_permission(permission=permission_acl_edit)

        response = self.post(
            viewname='acls:acl_create',
            kwargs=self.test_object_view_kwargs, data={
                'role': self.test_role.pk
            }, follow=True
        )
        self.assertNotContains(
            response=response, text='optgroup', status_code=200
        )

        self.assertEqual(AccessControlList.objects.count(), 1)



from django.contrib import messages
from django.db import transaction
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.events import event_document_type_edited
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.documents.permissions import permission_document_type_edit
from mayan.apps.views.generics import (
    AddRemoveView, ConfirmView, MultipleObjectFormActionView,
    MultipleObjectConfirmActionView, SingleObjectCreateView,
    SingleObjectDetailView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.views.mixins import ExternalObjectMixin

from ..events import event_workflow_edited
from ..forms import (
    WorkflowForm, WorkflowMultipleSelectionForm, WorkflowPreviewForm
)
from ..icons import icon_workflow_template_list
from ..links import link_workflow_template_create
from ..models import Workflow
from ..permissions import (
    permission_workflow_create, permission_workflow_delete,
    permission_workflow_edit, permission_workflow_tools,
    permission_workflow_view,
)
from ..tasks import (
    task_launch_all_workflows, task_launch_workflow, task_launch_workflow_for
)


class DocumentTypeWorkflowTemplatesView(AddRemoveView):
    main_object_permission = permission_document_type_edit
    main_object_model = DocumentType
    main_object_pk_url_kwarg = 'document_type_id'
    secondary_object_model = Workflow
    secondary_object_permission = permission_workflow_edit
    list_available_title = _('Available workflows')
    list_added_title = _('Workflows assigned this document type')
    related_field = 'workflows'

    def get_actions_extra_kwargs(self):
        return {'_user': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Removing a workflow from a document type will also '
                'remove all running instances of that workflow.'
            ),
            'title': _(
                'Workflows assigned the document type: %s'
            ) % self.main_object,
        }

    def action_add(self, queryset, _user):
        with transaction.atomic():
            event_document_type_edited.commit(
                actor=_user, target=self.main_object
            )

            for obj in queryset:
                self.main_object.workflows.add(obj)
                event_workflow_edited.commit(
                    action_object=self.main_object, actor=_user, target=obj
                )

    def action_remove(self, queryset, _user):
        with transaction.atomic():
            event_document_type_edited.commit(
                actor=_user, target=self.main_object
            )

            for obj in queryset:
                self.main_object.workflows.remove(obj)
                event_workflow_edited.commit(
                    action_object=self.main_object, actor=_user,
                    target=obj
                )
                obj.instances.filter(
                    document__document_type=self.main_object
                ).delete()


class DocumentWorkflowTemplatesLaunchView(MultipleObjectFormActionView):
    form_class = WorkflowMultipleSelectionForm
    model = Document
    object_permission = permission_workflow_tools
    pk_url_kwarg = 'document_id'
    success_message = _('Workflows launched for %(count)d document')
    success_message_plural = _('Workflows launched for %(count)d documents')

    def get_extra_context(self):
        result = {
            'submit_label': _('Launch'),
            'subtitle': _(
                'Workflows already launched or workflows not applicable to '
                'some documents when multiple documents are selected, '
                'will be silently ignored.'
            ),
            'title': ungettext(
                singular='Launch selected workflows for %(count)d document',
                plural='Launch selected workflows for %(count)d documents',
                number=self.object_list.count()
            ) % {
                'count': self.object_list.count(),
            }
        }

        if self.object_list.count() == 1:
            result.update(
                {
                    'object': self.object_list.first(),
                    'title': _(
                        'Launch selected workflows for document: %s'
                    ) % self.object_list.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        workflows_union = Workflow.objects.filter(
            document_types__in=self.object_list.values('document_type')
        ).distinct()

        result = {
            'help_text': _('Workflows to be launched.'),
            'permission': permission_workflow_tools,
            'queryset': workflows_union,
            'user': self.request.user
        }

        return result

    def object_action(self, form, instance):
        workflow_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_workflow_tools,
            queryset=form.cleaned_data['workflows'], user=self.request.user
        )

        for workflow in workflow_queryset:
            task_launch_workflow_for.apply_async(
                kwargs={
                    'document_id': instance.pk, 'workflow_id': workflow.pk
                }
            )


class WorkflowTemplateCreateView(SingleObjectCreateView):
    extra_context = {'title': _('Create workflow')}
    form_class = WorkflowForm
    model = Workflow
    post_action_redirect = reverse_lazy(
        viewname='document_states:workflow_template_list'
    )
    view_permission = permission_workflow_create

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class WorkflowTemplateDeleteView(MultipleObjectConfirmActionView):
    model = Workflow
    object_permission = permission_workflow_delete
    pk_url_kwarg = 'workflow_template_id'
    post_action_redirect = reverse_lazy(
        viewname='document_states:workflow_template_list'
    )
    success_message = _('Delete request performed on %(count)d workflow')
    success_message_plural = _(
        'Delete request performed on %(count)d workflows'
    )

    def get_extra_context(self):
        result = {
            'delete_view': True,
            'message': _('All workflow instances will also be deleted.'),
            'title': ungettext(
                singular='Delete the selected workflow?',
                plural='Delete the selected workflows?',
                number=self.object_list.count()
            )
        }

        if self.object_list.count() == 1:
            result.update(
                {
                    'object': self.object_list.first(),
                    'title': _('Delete workflow: %s?') % self.object_list.first()
                }
            )

        return result

    def object_action(self, instance, form=None):
        try:
            instance.delete()
            messages.success(
                message=_(
                    'Workflow "%s" deleted successfully.'
                ) % instance, request=self.request
            )
        except Exception as exception:
            messages.error(
                message=_('Error deleting workflow "%(workflow)s": %(error)s') % {
                    'workflow': instance, 'error': exception
                }, request=self.request
            )


class WorkflowTemplateEditView(SingleObjectEditView):
    form_class = WorkflowForm
    model = Workflow
    object_permission = permission_workflow_edit
    pk_url_kwarg = 'workflow_template_id'
    post_action_redirect = reverse_lazy(
        viewname='document_states:workflow_template_list'
    )

    def get_extra_context(self):
        return {
            'title': _(
                'Edit workflow: %s'
            ) % self.object,
        }

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class WorkflowTemplateDocumentTypesView(AddRemoveView):
    main_object_permission = permission_workflow_edit
    main_object_model = Workflow
    main_object_pk_url_kwarg = 'workflow_template_id'
    secondary_object_model = DocumentType
    secondary_object_permission = permission_document_type_edit
    list_available_title = _('Available document types')
    list_added_title = _('Document types assigned this workflow')
    related_field = 'document_types'

    def get_actions_extra_kwargs(self):
        return {'_user': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Removing a document type from a workflow will also '
                'remove all running instances of that workflow for '
                'documents of the document type just removed.'
            ),
            'title': _(
                'Document types assigned the workflow: %s'
            ) % self.main_object,
        }

    def action_add(self, queryset, _user):
        with transaction.atomic():
            event_workflow_edited.commit(
                actor=_user, target=self.main_object
            )

            for obj in queryset:
                self.main_object.document_types.add(obj)
                event_document_type_edited.commit(
                    action_object=self.main_object, actor=_user, target=obj
                )

    def action_remove(self, queryset, _user):
        with transaction.atomic():
            event_workflow_edited.commit(
                actor=_user, target=self.main_object
            )

            for obj in queryset:
                self.main_object.document_types.remove(obj)
                event_document_type_edited.commit(
                    action_object=self.main_object, actor=_user,
                    target=obj
                )
                self.main_object.instances.filter(
                    document__document_type=obj
                ).delete()


class WorkflowTemplateLaunchView(ExternalObjectMixin, ConfirmView):
    external_object_class = Workflow
    external_object_permission = permission_workflow_tools
    external_object_pk_url_kwarg = 'workflow_template_id'

    def get_extra_context(self):
        return {
            'title': _('Launch workflow?'),
            'subtitle': _(
                'This will launch the workflow for documents that have '
                'already been uploaded.'
            )
        }

    def view_action(self):
        task_launch_workflow.apply_async(
            kwargs={
                'workflow_id': self.external_object.pk,
            }
        )
        messages.success(
            message=_('Workflow launch queued successfully.'),
            request=self.request
        )


class WorkflowTemplateListView(SingleObjectListView):
    model = Workflow
    object_permission = permission_workflow_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_workflow_template_list,
            'no_results_main_link': link_workflow_template_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Workflows store a series of states and keep track of the '
                'current state of a document. Transitions are used to change the '
                'current state to a new one.'
            ),
            'no_results_title': _(
                'No workflows have been defined'
            ),
            'title': _('Workflows'),
        }


class WorkflowTemplatePreviewView(SingleObjectDetailView):
    form_class = WorkflowPreviewForm
    model = Workflow
    object_permission = permission_workflow_view
    pk_url_kwarg = 'workflow_template_id'

    def get_extra_context(self):
        return {
            'hide_labels': True,
            'object': self.object,
            'title': _('Preview of: %s') % self.object
        }


class ToolLaunchWorkflows(ConfirmView):
    extra_context = {
        'title': _('Launch all workflows?'),
        'subtitle': _(
            'This will launch all workflows created after documents have '
            'already been uploaded.'
        )
    }
    view_permission = permission_workflow_tools

    def view_action(self):
        task_launch_all_workflows.apply_async()
        messages.success(
            message=_('Workflow launch queued successfully.'),
            request=self.request
        )



from django.template import Context, Engine, Template as DjangoTemplate


class Template:
    def __init__(self, template_string):
        engine = Engine(
            builtins=[
                'mathfilters.templatetags.mathfilters',
                'mayan.apps.templating.templatetags.templating_tags',
            ]
        )

        self._template = DjangoTemplate(
            engine=engine, template_string=template_string
        )

    def render(self, context=None):
        context_object = Context(dict_=context or {})

        return self._template.render(context=context_object)



import os
import tempfile

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

namespace = SettingNamespace(label=_('Storage'), name='storage')

setting_shared_storage = namespace.add_setting(
    global_name='STORAGE_SHARED_STORAGE',
    default='django.core.files.storage.FileSystemStorage',
    help_text=_('A storage backend that all workers can use to share files.')
)
setting_shared_storage_arguments = namespace.add_setting(
    global_name='STORAGE_SHARED_STORAGE_ARGUMENTS',
    default={'location': os.path.join(settings.MEDIA_ROOT, 'shared_files')}
)
setting_temporary_directory = namespace.add_setting(
    global_name='STORAGE_TEMPORARY_DIRECTORY', default=tempfile.gettempdir(),
    help_text=_(
        'Temporary directory used site wide to store thumbnails, previews '
        'and temporary files.'
    )
)



import os
import time

from django.conf import settings

from mayan.apps.converter.classes import Layer
from mayan.apps.converter.layers import layer_saved_transformations

from ..literals import PAGE_RANGE_ALL
from ..models import Document, DocumentType, FavoriteDocument

from .literals import (
    TEST_DOCUMENT_DESCRIPTION_EDITED, TEST_DOCUMENT_PATH,
    TEST_DOCUMENT_TYPE_DELETE_PERIOD, TEST_DOCUMENT_TYPE_DELETE_TIME_UNIT,
    TEST_DOCUMENT_TYPE_LABEL, TEST_DOCUMENT_TYPE_LABEL_EDITED,
    TEST_DOCUMENT_TYPE_QUICK_LABEL, TEST_DOCUMENT_TYPE_QUICK_LABEL_EDITED,
    TEST_DOCUMENT_VERSION_COMMENT_EDITED, TEST_SMALL_DOCUMENT_FILENAME,
    TEST_SMALL_DOCUMENT_PATH, TEST_TRANSFORMATION_ARGUMENT,
    TEST_TRANSFORMATION_CLASS, TEST_VERSION_COMMENT
)


class DocumentAPIViewTestMixin:
    def _request_test_document_api_download_view(self):
        return self.get(
            viewname='rest_api:document-download', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_document_api_upload_view(self):
        with open(file=TEST_DOCUMENT_PATH, mode='rb') as file_object:
            return self.post(
                viewname='rest_api:document-list', data={
                    'document_type': self.test_document_type.pk,
                    'file': file_object
                }
            )

    def _request_test_document_description_api_edit_via_patch_view(self):
        return self.patch(
            viewname='rest_api:document-detail', kwargs={
                'pk': self.test_document.pk
            }, data={'description': TEST_DOCUMENT_DESCRIPTION_EDITED}
        )

    def _request_test_document_description_api_edit_via_put_view(self):
        return self.put(
            viewname='rest_api:document-detail', kwargs={
                'pk': self.test_document.pk
            }, data={'description': TEST_DOCUMENT_DESCRIPTION_EDITED}
        )

    def _request_test_document_document_type_change_api_view(self):
        return self.post(
            viewname='rest_api:document-type-change', kwargs={
                'pk': self.test_document.pk
            }, data={'new_document_type': self.test_document_type_2.pk}
        )


class DocumentPageAPIViewTestMixin:
    def _request_document_page_image(self):
        page = self.test_document.pages.first()
        return self.get(
            viewname='rest_api:documentpage-image', kwargs={
                'pk': page.document_version.document_id, 'version_pk': page.document_version_id,
                'page_pk': page.pk
            }
        )


class DocumentPageDisableViewTestMixin:
    def _disable_test_document_page(self):
        self.test_document_page.enabled = False
        self.test_document_page.save()

    def _request_test_document_page_disable_view(self):
        return self.post(
            viewname='documents:document_page_disable', kwargs={
                'document_page_id': self.test_document_page.pk
            }
        )

    def _request_test_document_page_enable_view(self):
        return self.post(
            viewname='documents:document_page_enable', kwargs={
                'document_page_id': self.test_document_page.pk
            }
        )

    def _request_test_document_page_multiple_disable_view(self):
        return self.post(
            viewname='documents:document_page_multiple_disable', data={
                'id_list': self.test_document_page.pk
            }
        )

    def _request_test_document_page_multiple_enable_view(self):
        return self.post(
            viewname='documents:document_page_multiple_enable', data={
                'id_list': self.test_document_page.pk
            }
        )


class DocumentPageViewTestMixin:
    def _request_test_document_page_list_view(self):
        return self.get(
            viewname='documents:document_pages', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_page_rotate_left_view(self):
        return self.post(
            viewname='documents:document_page_rotate_left', kwargs={
                'document_page_id': self.test_document_page.pk
            }
        )

    def _request_test_document_page_rotate_right_view(self):
        return self.post(
            viewname='documents:document_page_rotate_right', kwargs={
                'document_page_id': self.test_document_page.pk
            }
        )

    def _request_test_document_page_view(self, document_page):
        return self.get(
            viewname='documents:document_page_view', kwargs={
                'document_page_id': document_page.pk,
            }
        )

    def _request_test_document_page_zoom_in_view(self):
        return self.post(
            viewname='documents:document_page_zoom_in', kwargs={
                'document_page_id': self.test_document_page.pk
            }
        )

    def _request_test_document_page_zoom_out_view(self):
        return self.post(
            viewname='documents:document_page_zoom_out', kwargs={
                'document_page_id': self.test_document_page.pk
            }
        )


class DocumentQuickLabelViewTestMixin:
    def _request_document_quick_label_edit_view(self, extra_data=None):
        data = {
            'document_type_available_filenames': self.test_document_type_filename.pk,
            'label': ''
            # View needs at least an empty label for quick
            # label to work. Cause is unknown.
        }
        data.update(extra_data or {})

        return self.post(
            viewname='documents:document_edit', kwargs={
                'document_id': self.test_document.pk
            }, data=data
        )


class DocumentTestMixin:
    auto_create_test_document_type = True
    auto_upload_test_document = True
    test_document_filename = TEST_SMALL_DOCUMENT_FILENAME
    test_document_path = None

    def setUp(self):
        super(DocumentTestMixin, self).setUp()
        Layer.invalidate_cache()

        self.test_documents = []

        if self.auto_create_test_document_type:
            self._create_test_document_type()

            if self.auto_upload_test_document:
                self._upload_test_document()

    def tearDown(self):
        for document_type in DocumentType.objects.all():
            document_type.delete()
        super(DocumentTestMixin, self).tearDown()

    def _create_test_document_stub(self):
        self.test_document_stub = Document.objects.create(
            document_type=self.test_document_type, label='document_stub'
        )

    def _create_test_document_type(self):
        self.test_document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_LABEL
        )

    def _calculate_test_document_path(self):
        if not self.test_document_path:
            self.test_document_path = os.path.join(
                settings.BASE_DIR, 'apps', 'documents', 'tests', 'contrib',
                'sample_documents', self.test_document_filename
            )

    def _upload_test_document(self, label=None, _user=None):
        self._calculate_test_document_path()

        if not label:
            label = self.test_document_filename

        with open(file=self.test_document_path, mode='rb') as file_object:
            self.test_document, self.test_document_version = self.test_document_type.new_document(
                file_object=file_object, label=label, _user=_user
            )

        self.test_documents.append(self.test_document)
        self.test_document_page = self.test_document_version.pages.first()


class DocumentTypeAPIViewTestMixin:
    def _request_test_document_type_api_create_view(self):
        return self.post(
            viewname='rest_api:documenttype-list', data={
                'label': TEST_DOCUMENT_TYPE_LABEL
            }
        )

    def _request_test_document_type_api_delete_view(self):
        return self.delete(
            viewname='rest_api:documenttype-detail', kwargs={
                'pk': self.test_document_type.pk,
            }
        )

    def _request_test_document_type_api_patch_view(self):
        return self.patch(
            viewname='rest_api:documenttype-detail', kwargs={
                'pk': self.test_document_type.pk,
            }, data={'label': TEST_DOCUMENT_TYPE_LABEL_EDITED}
        )

    def _request_test_document_type_api_put_view(self):
        return self.put(
            viewname='rest_api:documenttype-detail', kwargs={
                'pk': self.test_document_type.pk,
            }, data={'label': TEST_DOCUMENT_TYPE_LABEL_EDITED}
        )


class DocumentTypeDeletionPoliciesViewTestMixin:
    def _request_document_type_filename_generator_get_view(self):
        return self.get(
            viewname='documents:document_type_policies', kwargs={
                'document_type_id': self.test_document_type.pk
            }
        )


class DocumentTypeFilenameGeneratorViewTestMixin:
    def _request_document_type_filename_generator_get_view(self):
        return self.get(
            viewname='documents:document_type_filename_generator', kwargs={
                'document_type_id': self.test_document_type.pk
            }
        )


class DocumentTypeQuickLabelViewTestMixin:
    def _request_quick_label_create(self):
        return self.post(
            viewname='documents:document_type_filename_create', kwargs={
                'document_type_id': self.test_document_type.pk
            }, data={
                'filename': TEST_DOCUMENT_TYPE_QUICK_LABEL,
            }
        )

    def _request_quick_label_delete(self):
        return self.post(
            viewname='documents:document_type_filename_delete', kwargs={
                'document_type_filename_id': self.test_document_type_filename.pk
            }
        )

    def _request_quick_label_edit(self):
        return self.post(
            viewname='documents:document_type_filename_edit', kwargs={
                'document_type_filename_id': self.test_document_type_filename.pk
            }, data={
                'filename': TEST_DOCUMENT_TYPE_QUICK_LABEL_EDITED,
            }
        )

    def _request_quick_label_list_view(self):
        return self.get(
            viewname='documents:document_type_filename_list', kwargs={
                'document_type_id': self.test_document_type.pk
            }
        )


class DocumentTypeQuickLabelTestMixin:
    def _create_test_quick_label(self):
        self.test_document_type_filename = self.test_document_type.filenames.create(
            filename=TEST_DOCUMENT_TYPE_QUICK_LABEL
        )


class DocumentVersionAPIViewTestMixin:
    def _request_test_document_version_api_download_view(self):
        return self.get(
            viewname='rest_api:documentversion-download', kwargs={
                'pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk,
            }
        )

    def _request_test_document_version_api_edit_via_patch_view(self):
        return self.patch(
            viewname='rest_api:documentversion-detail', kwargs={
                'pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk
            }, data={'comment': TEST_DOCUMENT_VERSION_COMMENT_EDITED}
        )

    def _request_test_document_version_api_edit_via_put_view(self):
        return self.put(
            viewname='rest_api:documentversion-detail', kwargs={
                'pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk
            }, data={'comment': TEST_DOCUMENT_VERSION_COMMENT_EDITED}
        )

    def _request_test_document_version_api_list_view(self):
        return self.get(
            viewname='rest_api:document-version-list', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_document_version_api_revert_view(self):
        return self.delete(
            viewname='rest_api:documentversion-detail', kwargs={
                'pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk
            }
        )

    def _request_test_document_version_api_upload_view(self):
        # Artificial delay since MySQL doesn't store microsecond data in
        # timestamps. Version timestamp is used to determine which version
        # is the latest.
        time.sleep(1)

        with open(file=TEST_DOCUMENT_PATH, mode='rb') as file_descriptor:
            return self.post(
                viewname='rest_api:document-version-list', kwargs={
                    'pk': self.test_document.pk,
                }, data={
                    'comment': '', 'file': file_descriptor,
                }
            )


class DocumentVersionTestMixin:
    def _upload_new_version(self):
        with open(file=TEST_SMALL_DOCUMENT_PATH, mode='rb') as file_object:
            self.test_document.new_version(
                comment=TEST_VERSION_COMMENT, file_object=file_object
            )


class DocumentVersionViewTestMixin:
    def _request_document_version_download(self, data=None):
        data = data or {}
        return self.get(
            viewname='documents:document_version_download', kwargs={
                'document_version_id': self.test_document.latest_version.pk
            }, data=data
        )

    def _request_document_version_list_view(self):
        return self.get(
            viewname='documents:document_version_list', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_document_version_revert_view(self, document_version):
        return self.post(
            viewname='documents:document_version_revert', kwargs={
                'document_version_id': document_version.pk
            }
        )


class DocumentViewTestMixin:
    def _create_document_transformation(self):
        layer_saved_transformations.add_transformation_to(
            obj=self.test_document.pages.first(),
            transformation_class=TEST_TRANSFORMATION_CLASS,
            arguments=TEST_TRANSFORMATION_ARGUMENT
        )

    def _request_test_document_clear_transformations_view(self):
        return self.post(
            viewname='documents:document_clear_transformations',
            kwargs={'document_id': self.test_document.pk}
        )

    def _request_test_document_download_form_get_view(self):
        return self.get(
            viewname='documents:document_download_form', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_download_form_post_view(self):
        return self.post(
            viewname='documents:document_download_form', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_download_view(self):
        return self.get(
            viewname='documents:document_download', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_multiple_download_view(self):
        return self.get(
            viewname='documents:document_multiple_download',
            data={'id_list': self.test_document.pk}
        )

    def _request_test_document_list_view(self):
        return self.get(viewname='documents:document_list')

    def _request_test_document_preview_view(self):
        return self.get(
            viewname='documents:document_preview', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_print_view(self):
        return self.get(
            viewname='documents:document_print', kwargs={
                'document_id': self.test_document.pk,
            }, data={
                'page_group': PAGE_RANGE_ALL
            }
        )

    def _request_test_document_properties_view(self):
        return self.get(
            viewname='documents:document_properties', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_properties_edit_get_view(self):
        return self.get(
            viewname='documents:document_edit', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_type_edit_get_view(self):
        return self.get(
            viewname='documents:document_document_type_edit', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_type_edit_post_view(self, document_type):
        return self.post(
            viewname='documents:document_document_type_edit', kwargs={
                'document_id': self.test_document.pk
            }, data={'document_type': document_type.pk}
        )

    def _request_test_document_multiple_type_edit(self, document_type):
        return self.post(
            viewname='documents:document_multiple_document_type_edit',
            data={
                'id_list': self.test_document.pk,
                'document_type': document_type.pk
            }
        )

    def _request_test_document_update_page_count_view(self):
        return self.post(
            viewname='documents:document_update_page_count',
            kwargs={'document_id': self.test_document.pk}
        )

    def _request_test_document_multiple_update_page_count_view(self):
        return self.post(
            viewname='documents:document_multiple_update_page_count',
            data={'id_list': self.test_document.pk}
        )

    def _request_test_document_multiple_clear_transformations(self):
        return self.post(
            viewname='documents:document_multiple_clear_transformations',
            data={'id_list': self.test_document.pk}
        )

    def _request_empty_trash_view(self):
        return self.post(viewname='documents:trash_can_empty')


class DocumentTypeViewTestMixin:
    def _request_test_document_type_create_view(self):
        return self.post(
            viewname='documents:document_type_create',
            data={
                'label': TEST_DOCUMENT_TYPE_LABEL,
                'delete_time_period': TEST_DOCUMENT_TYPE_DELETE_PERIOD,
                'delete_time_unit': TEST_DOCUMENT_TYPE_DELETE_TIME_UNIT
            }
        )

    def _request_test_document_type_delete_view(self):
        return self.post(
            viewname='documents:document_type_delete', kwargs={
                'document_type_id': self.test_document_type.pk
            }
        )

    def _request_test_document_type_edit_view(self):
        return self.post(
            viewname='documents:document_type_edit', kwargs={
                'document_type_id': self.test_document_type.pk
            }, data={
                'label': TEST_DOCUMENT_TYPE_LABEL_EDITED,
            }
        )

    def _request_test_document_type_list_view(self):
        return self.get(viewname='documents:document_type_list')


class DuplicatedDocumentsViewsTestMixin:
    def _request_document_duplicates_list_view(self):
        return self.get(
            viewname='documents:document_duplicates_list', kwargs={
                'document_id': self.test_documents[0].pk
            }
        )

    def _request_duplicated_document_list_view(self):
        return self.get(viewname='documents:duplicated_document_list')

    def _upload_duplicate_document(self):
        self._upload_test_document()


class FavoriteDocumentsTestMixin:
    def _request_document_add_to_favorites_view(self):
        return self.post(
            viewname='documents:document_add_to_favorites',
            kwargs={'document_id': self.test_document.pk}
        )

    def _document_add_to_favorites(self):
        FavoriteDocument.objects.add_for_user(
            document=self.test_document, user=self._test_case_user
        )

    def _request_document_list_favorites(self):
        return self.get(
            viewname='documents:document_list_favorites',
        )

    def _request_document_remove_from_favorites(self):
        return self.post(
            viewname='documents:document_remove_from_favorites',
            kwargs={'document_id': self.test_document.pk}
        )


class TrashedDocumentAPIViewTestMixin:
    def _request_test_document_api_trash_view(self):
        return self.delete(
            viewname='rest_api:document-detail', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_trashed_document_api_delete_view(self):
        return self.delete(
            viewname='rest_api:trasheddocument-detail', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_trashed_document_api_detail_view(self):
        return self.get(
            viewname='rest_api:trasheddocument-detail', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_trashed_document_api_image_view(self):
        latest_version = self.test_document.latest_version

        return self.get(
            viewname='rest_api:documentpage-image', kwargs={
                'pk': latest_version.document_id,
                'version_pk': latest_version.pk,
                'page_pk': latest_version.pages.first().pk
            }
        )

    def _request_test_trashed_document_api_list_view(self):
        return self.get(
            viewname='rest_api:trasheddocument-list'
        )

    def _request_test_trashed_document_api_restore_view(self):
        return self.post(
            viewname='rest_api:trasheddocument-restore', kwargs={
                'pk': self.test_document.pk
            }
        )


class TrashedDocumentViewTestMixin:
    def _request_document_trash_get_view(self):
        return self.get(
            viewname='documents:document_trash', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_document_trash_post_view(self):
        return self.post(
            viewname='documents:document_trash', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_trashed_document_restore_get_view(self):
        return self.get(
            viewname='documents:document_restore', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_trashed_document_restore_post_view(self):
        return self.post(
            viewname='documents:document_restore', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_trashed_document_delete_get_view(self):
        return self.get(
            viewname='documents:document_delete', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_trashed_document_delete_post_view(self):
        return self.post(
            viewname='documents:document_delete', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_trashed_document_list_view(self):
        return self.get(viewname='documents:document_list_deleted')
